@@ -7,6 +7,10 @@ import dill
 import os
 from joblib import dump
 import zipfile
+
+from tqdm import tqdm
+from tqdm.utils import CallbackIOWrapper
+
 from .utils import zipdir, getApiKey, generate_requirements_file, auth, get_model, new_version, deploy_version, find_version
 
 api_host = 'https://api.inferrd.com'
@@ -47,23 +51,19 @@ def get_requests(name, **kwargs):
 
   url = api_host + '/service/' + model['id'] + '/requests?' + ('responseStatus=200&' if not includeFailures else '') + 'limit=' + str(limit) + '&page=' + str(page) + ('&version=' + version if version else '')
 
-  print(url)
-
   r = requests.get(url, headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
 
   return r.json()
 
 def deploy(model, **kwargs):
   if(getApiKey() == ''):
-    print('No api key. Use inferrd.auth() first.')
-    exit()
+    raise Exception('No api key. Use inferrd.auth() first.')
 
   prediction_fn = model
   name = kwargs['name']
   
   if prediction_fn is None:
-    print('Empty function model. Make sure the first argument is a function.')
-    exit()
+    raise Exception('Empty function model. Make sure the first argument is a function.')
 
   model = get_model(name)
 
@@ -84,12 +84,23 @@ def deploy(model, **kwargs):
   zipf = zipfile.ZipFile('model.zip', 'w', zipfile.ZIP_DEFLATED)
   zipf.write('./model.dill', './model.dill')
   zipf.write('./reqs.txt', './requirements.txt')
+
+  if 'setup' in kwargs:
+    setupFile = open('./setup.sh', 'w')
+    setupFile.write(kwargs['setup'])
+    setupFile.close()
+    zipf.write('./setup.sh', './setup.sh')
+
   zipf.close()
   
   # upload to storage
-  print('> Uploading model')
+  model_size = Path('./model.zip').stat().st_size
+  print('> Uploading model ({0}MB)'.format(round(model_size / 1024 / 1024, 3)))
   f = open("./model.zip", 'rb')
-  r = requests.put(version['signedUpload'], data=f, headers={'Content-Type': 'application/zip'})
+
+  with tqdm(total=model_size, unit="B", unit_scale=True, unit_divisor=1024, ncols=100) as t:
+    wrapped_file = CallbackIOWrapper(t.update, f, "read")
+    r = requests.put(version['signedUpload'], data=wrapped_file, headers={'Content-Type': 'application/zip'})
 
   print('> Deploying version v' + str(version['number']))
   deploy_version(version['id'])
@@ -98,22 +109,22 @@ def deploy(model, **kwargs):
   os.remove('./model.dill')
   os.remove('./reqs.txt')
 
+  if 'setup' in kwargs:
+    os.remove('./setup.sh')
+
   print('Your model is now deploying! Try it out using this code:\n')
   print('inferrd.get(\'{0}\')([1,2])'.format(name))
-  return version['number']
 
 
 # ------ TENSORFLOW
 def deploy_tf(tf_model, name):
   if(getApiKey() == ''):
-    print('No api key. Use inferrd.auth() first.')
-    exit()
+    raise Exception('No api key. Use inferrd.auth() first.')
 
   print('> Saving model to folder')
 
   if tf_model is None:
-    print('Empty tensorflow model. Make sure the first argument is a TensorFlow v2 model.')
-    exit()
+    raise Exception('Empty tensorflow model. Make sure the first argument is a TensorFlow v2 model.')
 
   model = get_model(name)
 
@@ -152,14 +163,12 @@ def deploy_tf(tf_model, name):
 # ------ SCIKIT
 def deploy_scikit(scikit_model, name):
   if(getApiKey() == ''):
-    print('No api key. Use inferrd.auth() first.')
-    exit()
+    raise Exception('No api key. Use inferrd.auth() first.')
 
   print('> Saving model to folder')
 
   if scikit_model is None:
-    print('Empty Scikit model. Make sure the first argument is a Scikit Learn model.')
-    exit()
+    raise Exception('Empty Scikit model. Make sure the first argument is a Scikit Learn model.')
 
   model = get_model(name)
 
@@ -196,14 +205,12 @@ def deploy_scikit(scikit_model, name):
 # ------ SPACY
 def deploy_spacy(nlp_model, name):
   if(getApiKey() == ''):
-    print('No api key. Use inferrd.auth() first.')
-    exit()
+    raise Exception('No api key. Use inferrd.auth() first.')
 
   print('> Saving model to folder')
 
   if nlp_model is None:
-    print('Empty spaCy model. Make sure the first argument is a spaCy model.')
-    exit()
+    raise Exception('Empty spaCy model. Make sure the first argument is a spaCy model.')
 
   model = get_model(name)
 
